<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

class reviDatabaseModel
{
    var $prefix;
    var $wpdb;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->prefix = $this->wpdb->prefix;
    }

    /* Verificar si las nuevas tablas de WooCommerce existen */
    public function checkNewWoocommerceTables()
    {
        if ($this->tableExistsAndHasValues('wc_orders')) {
            return true;
        }
        return false;
    }

    public function tableExistsAndHasValues($tableName)
    {
        $tableExists = $this->tableExists($tableName);
        if ($tableExists) {
            $orders_exist = $this->wpdb->get_var("SELECT COUNT(*) FROM {$this->prefix}{$tableName}");
            if ($orders_exist > 0) {
                return true;
            }
        }
        return false;
    }

    public function tableExists($tableName)
    {
        // Verificar tablas usando INFORMATION_SCHEMA
        $result = $this->wpdb->get_var(
            $this->wpdb->prepare(
                "SELECT TABLE_NAME 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s",
                DB_NAME,
                $this->prefix . $tableName
            )
        );

        return $result !== null;
    }

    public function columnExists($tableName, $columnName)
    {
        $result = $this->wpdb->get_var(
            $this->wpdb->prepare(
                "SELECT COLUMN_NAME
             FROM INFORMATION_SCHEMA.COLUMNS 
             WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s AND COLUMN_NAME = %s",
                DB_NAME,
                $this->prefix . $tableName,
                $columnName
            )
        );

        return $result !== null;
    }

    function checkAndCreateReviTables()
    {
        // Asegurarse de que el entorno de administración de WordPress esté cargado
        if (! function_exists('dbDelta')) {
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
        }

        // Upgrade si es necesario
        $this->update_revi_database_from_files();

        // Obtener el conjunto de caracteres de la base de datos
        $charset_collate = $this->wpdb->get_charset_collate();

        if (!$this->tableExists('revi_orders')) {
            $sql = "CREATE TABLE IF NOT EXISTS `" . $this->prefix . "revi_orders` (
                `id_order` BIGINT(20) NOT NULL,
                `date_sent` DATETIME NOT NULL,
                PRIMARY KEY (`id_order`)
            ) $charset_collate;";

            dbDelta($sql);
        }

        if (!$this->tableExists('revi_products')) {
            $sql = "CREATE TABLE IF NOT EXISTS `" . $this->prefix . "revi_products` (
            `id_product` VARCHAR(36) NOT NULL,
            `num_reviews` INT(10) UNSIGNED NOT NULL,
            `avg_rating` FLOAT(5) NOT NULL,
            `date_sent` DATETIME NOT NULL,
            PRIMARY KEY (`id_product`)
        ) $charset_collate;";

            dbDelta($sql);
        }
    }

    function deleteReviDatabase()
    {
        $tables = ['revi_orders', 'revi_products', 'revi_comments', 'revi_categories'];

        foreach ($tables as $table) {
            $tableName = esc_sql($table);
            $sql = 'DROP TABLE IF EXISTS `' . $this->prefix . $tableName . '`';
            $this->wpdb->query($sql);
        }
    }

    public function truncateReviDatabase()
    {
        try {
            $this->truncateOrders();
            $this->truncateProducts();

            echo "\nAll Revi Data Tables reset";
        } catch (Exception $e) {
            echo "\nError truncateReviDatabase: " . $e->getMessage();
        }
    }

    public function truncateOrders()
    {
        $tableName = $this->prefix . 'revi_orders';

        // Comprobar si la tabla existe antes de intentar eliminar datos
        if ($this->wpdb->get_var($this->wpdb->prepare("SHOW TABLES LIKE %s", $tableName))) {
            $this->wpdb->query("TRUNCATE TABLE {$tableName}"); // Truncar tabla para eliminar todos los datos
        }

        wp_cache_delete('revi_sync_data');

        echo "<br>" . esc_html__('All Revi Orders Deleted', 'revi-io-customer-and-product-reviews') . "<br>"; // Escapado y traducción
    }

    public function truncateProducts()
    {
        $tableName = $this->prefix . 'revi_products';

        // Comprobar si la tabla existe antes de intentar eliminar datos
        if ($this->wpdb->get_var($this->wpdb->prepare("SHOW TABLES LIKE %s", $tableName))) {
            $this->wpdb->query("TRUNCATE TABLE {$tableName}"); // Truncar tabla para eliminar todos los datos
        }

        wp_cache_delete('revi_sync_data');

        echo "<br>" . esc_html__('All Revi Products Deleted', 'revi-io-customer-and-product-reviews') . "<br>"; // Escapado y traducción
    }

    function update_revi_database_from_files()
    {
        $upgrade_path = REVI_DIR . 'upgrade/';
        $module_version_bd = get_option('REVI_LAST_UPGRADE_VERSION');
        if (!isset($module_version_bd)) {
            $module_version_bd = '4.0.0';
        }


        // Lista de archivos de actualización en la carpeta upgrades
        $upgrade_files = [
            // '4.2.7' => 'upgrade-4.2.7.sql',
            // '6.0.7' => 'upgrade-6.0.7.sql',
        ];

        foreach ($upgrade_files as $version => $file) {
            if (version_compare($version, $module_version_bd, '>')) {
                $sql_file = $upgrade_path . $file;


                if (file_exists($sql_file)) {
                    $sql = file_get_contents($sql_file);

                    if ($sql) {
                        $sql_statements = explode(';', $sql);
                        foreach ($sql_statements as $statement) {
                            $statement = trim($statement);
                            if (!empty($statement)) {
                                $result = $this->wpdb->query($statement);
                                if ($result === false) {
                                    echo "Error executing SQL: " . esc_html($statement) . "Error: " . esc_html($this->wpdb->last_error);
                                } else {
                                    update_option('REVI_LAST_UPGRADE_VERSION', $version);
                                }
                            }
                        }
                    } else {
                        echo "Error reading SQL file: " . esc_html($sql_file);
                    }
                } else {
                    echo "SQL file not found: " . esc_html($sql_file);
                }
            }
        }
    }
}
